use std::sync::Arc;

use potato_data::{datapack::Datapack, registry::Registries};

pub struct Server {
    pub registries: Arc<Registries>,
}

macro_rules! print_registry_loaded {
    ($registry:expr, $name:literal) => {
        println!("Loaded {} {}s", $registry.len(), $name);
        println!("Loaded {} {} tags", $registry.tags_len(), $name);
    };
}

impl Server {
    #[allow(clippy::new_without_default)]
    pub fn new() -> Server {
        println!("Looking for datapacks...");
        let datapacks: Vec<_> = std::fs::read_dir("datapacks")
            .expect("Failed to read datapacks directory")
            .map(|f| f.unwrap().path())
            .filter(|p| p.is_dir())
            .map(Datapack::new)
            .collect();
        println!("Found {} datapacks", datapacks.len());
        println!("Loading datapacks...");
        let mut registries = Registries::empty();
        datapacks
            .iter()
            .for_each(|pack| pack.load(&mut registries).expect("Failed to load datapack"));

        println!("Finished loading datapacks");
        print_registry_loaded!(registries.trim_materials, "trim material");
        print_registry_loaded!(registries.trim_patterns, "trim pattern");
        print_registry_loaded!(registries.banner_patterns, "banner pattern");
        print_registry_loaded!(registries.chat_types, "chat type");
        print_registry_loaded!(registries.dimension_types, "dimension type");
        print_registry_loaded!(registries.painting_variants, "painting variant");
        print_registry_loaded!(registries.wolf_variants, "wolf variant");
        print_registry_loaded!(registries.damage_types, "damage_type");
        print_registry_loaded!(registries.worldgen.biomes, "biome");

        Server {
            registries: Arc::new(registries),
        }
    }
}
